@@ -14,12 +14,6 @@
 #include "soup-content-sniffer-stream.h"
 #include "soup.h"
 
-static void soup_content_sniffer_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
-
-G_DEFINE_TYPE_WITH_CODE (SoupContentSnifferStream, soup_content_sniffer_stream, G_TYPE_FILTER_INPUT_STREAM,
-			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
-						soup_content_sniffer_stream_pollable_init))
-
 enum {
 	PROP_0,
 
@@ -40,6 +34,13 @@ struct _SoupContentSnifferStreamPrivate {
 	GHashTable *sniffed_params;
 };
 
+static void soup_content_sniffer_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
+
+G_DEFINE_TYPE_WITH_CODE (SoupContentSnifferStream, soup_content_sniffer_stream, G_TYPE_FILTER_INPUT_STREAM,
+                         G_ADD_PRIVATE (SoupContentSnifferStream)
+			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
+						soup_content_sniffer_stream_pollable_init))
+
 static void
 soup_content_sniffer_stream_finalize (GObject *object)
 {
@@ -166,7 +167,8 @@ read_internal (GInputStream  *stream,
 
 	if (sniffer->priv->buffer) {
 		nread = MIN (count, sniffer->priv->buffer_nread);
-		memcpy (buffer, sniffer->priv->buffer, nread);
+		if (buffer)
+			memcpy (buffer, sniffer->priv->buffer, nread);
 		if (nread == sniffer->priv->buffer_nread) {
 			g_free (sniffer->priv->buffer);
 			sniffer->priv->buffer = NULL;
@@ -288,9 +290,7 @@ soup_content_sniffer_stream_create_source (GPollableInputStream *stream,
 static void
 soup_content_sniffer_stream_init (SoupContentSnifferStream *sniffer)
 {
-	sniffer->priv = G_TYPE_INSTANCE_GET_PRIVATE (sniffer,
-						     SOUP_TYPE_CONTENT_SNIFFER_STREAM,
-						     SoupContentSnifferStreamPrivate);
+	sniffer->priv = soup_content_sniffer_stream_get_instance_private (sniffer);
 	sniffer->priv->sniffing = TRUE;
 }
 
@@ -301,8 +301,6 @@ soup_content_sniffer_stream_class_init (SoupContentSnifferStreamClass *sniffer_c
 	GInputStreamClass *input_stream_class =
 		G_INPUT_STREAM_CLASS (sniffer_class);
  
-	g_type_class_add_private (sniffer_class, sizeof (SoupContentSnifferStreamPrivate));
-
 	object_class->finalize = soup_content_sniffer_stream_finalize;
 	object_class->set_property = soup_content_sniffer_stream_set_property;
 	object_class->get_property = soup_content_sniffer_stream_get_property;
