@@ -188,7 +188,7 @@ soup_date_new (int year, int month, int day,
  * offset_seconds is 0, returns the current time.
  *
  * If @offset_seconds would indicate a time not expressible as a
- * #time_t, the return value will be clamped into range.
+ * <type>time_t</type>, the return value will be clamped into range.
  *
  * Return value: a new #SoupDate
  **/
@@ -466,8 +466,8 @@ parse_textual_date (SoupDate *date, const char *date_string)
  * and reasonable approximations thereof. (Eg, it is lenient about
  * whitespace, leading "0"s, etc.)
  *
- * Return value: a new #SoupDate, or %NULL if @date_string could not
- * be parsed.
+ * Return value: (nullable): a new #SoupDate, or %NULL if @date_string
+ * could not be parsed.
  **/
 SoupDate *
 soup_date_new_from_string (const char *date_string)
@@ -525,7 +525,7 @@ soup_date_new_from_string (const char *date_string)
 
 /**
  * soup_date_new_from_time_t:
- * @when: a #time_t
+ * @when: a <type>time_t</type>
  *
  * Creates a #SoupDate corresponding to @when
  *
@@ -680,42 +680,46 @@ soup_date_to_string (SoupDate *date, SoupDateFormat format)
  * soup_date_to_time_t:
  * @date: a #SoupDate
  *
- * Converts @date to a %time_t.
+ * Converts @date to a <type>time_t</type>, assumming it to be in
+ * UTC.
  *
- * If @date is not representable as a %time_t, it will be clamped into
- * range. (In particular, some HTTP cookies have expiration dates
- * after "Y2.038k" (2038-01-19T03:14:07Z).)
+ * If @date is not representable as a <type>time_t</type>, it will be
+ * clamped into range. (In particular, some HTTP cookies have
+ * expiration dates after "Y2.038k" (2038-01-19T03:14:07Z).)
  *
- * Return value: @date as a %time_t
+ * Return value: @date as a <type>time_t</type>
  **/
 time_t
 soup_date_to_time_t (SoupDate *date)
 {
-	time_t tt;
-	GTimeVal val;
+	GDateTime *datetime;
+	gint64 seconds;
 
 	g_return_val_if_fail (date != NULL, 0);
 
-	/* FIXME: offset, etc */
-
 	if (date->year < 1970)
 		return 0;
 
 	/* If the year is later than 2038, we're guaranteed to
 	 * overflow a 32-bit time_t. (If it's exactly 2038, we'll
 	 * *probably* overflow, but only by a little, and it's easiest
-	 * to test that at the end by seeing if the result has turned
-	 * negative.)
+	 * to just clamp down the value if it's above G_MAXINT32.
 	 */
 	if (sizeof (time_t) == 4 && date->year > 2038)
-		return (time_t)0x7fffffff;
+		return (time_t)G_MAXINT32;
 
-	soup_date_to_timeval (date, &val);
-	tt = val.tv_sec;
+	datetime = g_date_time_new_utc (date->year,
+					date->month,
+					date->day,
+					date->hour,
+					date->minute,
+					date->second);
 
-	if (sizeof (time_t) == 4 && tt < 0)
-		return (time_t)0x7fffffff;
-	return tt;
+	seconds = g_date_time_to_unix (datetime);
+
+	g_date_time_unref (datetime);
+
+	return (time_t) (sizeof (time_t) == 4 ? MIN(seconds, G_MAXINT32) : seconds);
 }
 
 /**
